#!/usr/bin/php -q
<?php

    /*  Adrian SIMA, 2019
     *  dan.adrian@gmail.com
     */

    /* todo
      polling time for each device
     */

    error_reporting(0);

    /* get access to the database */
    include(dirname(__FILE__) . '/include/cli_check.php');

    /*  Reading ONT indexes is a cumbersome process, which can take longer than a decent
        amount of time expected for this operation. This is not exactly ok in large networks.
        At the same time, starting from the fact that a relevant variation measured values
        are significant at a rather large time interval, respectively several hours, I consider
        that reading the nodes distributed over a longer period of time is the solution
        for obtaining values, sufficiently relevant and without stressing the devices.

        From my tests I noticed that the querying a value took about 4 ms, more or less.
        Each ONU we have for now 5 values to read so thats mean 2 seconds for each set.
        Thus, at 500 ONT we will need 1000 seconds distributed in 12 hourly cycles ((2*500)/(60/5)) = ~80 sec,
        where 60 are $periodReadingTime and 5 minutes are crontab running interval.

        Keep in mind! This poller must be run as separated process from Cacti poller.
     */

    // minutes
    $periodReadingTime = 60;

    if (!db_table_exists('ont_opl_values')) {
        db_execute("CREATE TABLE `ont_opl_values` (
                `id` int(11) NOT NULL,
                `index` varchar(50) NOT NULL,
                `device` varchar(150) NOT NULL,
                `time_updated` datetime NOT NULL,
                `descr` varchar(250) NOT NULL,
                `onttx` float(4,2) NOT NULL,
                `ontrx` float(4,2) NOT NULL,
                `onttvrx` float(4,2) NOT NULL,
                `oltrx` float(4,2) NOT NULL
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        db_execute("ALTER TABLE `ont_opl_values`
                ADD PRIMARY KEY (`id`),
                ADD UNIQUE KEY `id_2` (`id`),
                ADD KEY `id` (`id`);");
        db_execute("ALTER TABLE `ont_opl_values`
                MODIFY `id` int(11) NOT NULL AUTO_INCREMENT;");
    }
    if (!db_table_exists('ont_opl_devices')) {
        db_execute("CREATE TABLE `ont_opl_devices` (
                `device` varchar(150) NOT NULL,
                `community` varchar(150) NOT NULL,
                `pollingtime` varchar(30) NOT NULL
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
    }

    $query_result = db_fetch_assoc("SELECT * FROM `ont_opl_devices`;");

    $mon_devices = [];
    $dev_com = [];

    foreach ($query_result as $device) {
        $mon_devices[] = [
            'hostname' => $device['device'],
            'community' => $device['community']
        ];

        if (!isset($dev_com[$device['device']])) {
            $dev_com[$device['device']] = $device['community'];
        }
    }

    $oidIndexes = [];
    foreach ($mon_devices as $device) {
        $snmpONTInterfaces = shell_exec("snmpwalk -OnE -v 2c {$device['hostname']} -c {$device['community']} .1.3.6.1.4.1.2011.6.128.1.1.2.43.1.9");
        $snmpONTInterface_ar = explode("\n", $snmpONTInterfaces);

        foreach ($snmpONTInterface_ar as $value) {
            $snmpONT_OID_ar = explode("=", $value);
            if (preg_match("/^.*([0-9]{10,20}\.[0-9]{1,3})/", $snmpONT_OID_ar[0], $oid_regex)) {
                if ($value) {
                    $oidIndexes[] = [
                        'index' => $oid_regex[1],
                        'hostname' => $device['hostname'],
                        'community' => $device['community']
                    ];
                }
            }
        }
    }

    $query_result_ar = db_fetch_assoc("SELECT * FROM `ont_opl_values`;");

    foreach ($oidIndexes as $device_values) {
        $insertIndex = true;

        foreach ($query_result_ar as $dbquery_values) {
            if ($dbquery_values['index'] === $device_values['index'] && $dbquery_values['device'] == $device_values['hostname']) {
                $insertIndex = false;
                break;
            }
        }

        if ($insertIndex) {
            db_execute("INSERT INTO `ont_opl_values` (`index`,`device`)
                VALUES ('{$device_values['index']}','{$device_values['hostname']}');");
        }
    }

    $updateLimits = cacti_sizeof($query_result_ar) / ($periodReadingTime / 5) * 1.20;

    // turning minutes into seconds
    $periodReadingTime *= 60;

    $query_result = db_fetch_assoc("SELECT * FROM `ont_opl_values`
                WHERE `time_updated` <= '" . date("Y-m-d H:i:s", (time() - $periodReadingTime)) . "'
                LIMIT 0," . (int)$updateLimits . ";");

    foreach ($query_result as $device_values) {
        $onttx   = shell_exec("snmpwalk -v 2c -Oqv {$device_values['device']} -c {$dev_com[$device_values['device']]} .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.3.{$device_values['index']}");
        $ontrx   = shell_exec("snmpwalk -v 2c -Oqv {$device_values['device']} -c {$dev_com[$device_values['device']]} .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.4.{$device_values['index']}");
        $onttvrx = shell_exec("snmpwalk -v 2c -Oqv {$device_values['device']} -c {$dev_com[$device_values['device']]} .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.7.{$device_values['index']}");
        $oltrx   = shell_exec("snmpwalk -v 2c -Oqv {$device_values['device']} -c {$dev_com[$device_values['device']]} .1.3.6.1.4.1.2011.6.128.1.1.2.51.1.6.{$device_values['index']}");
        $descr   = shell_exec("snmpwalk -v 2c -Oqv {$device_values['device']} -c {$dev_com[$device_values['device']]} .1.3.6.1.4.1.2011.6.128.1.1.2.43.1.9.{$device_values['index']}");

        db_execute("UPDATE `ont_opl_values`
                    SET `time_updated`='" . date("Y-m-d H:i:s") . "',
                        `descr`   = '" . str_replace('"', '', trim($descr)) . "',
                        `onttx`   = '" . (($onttx) / 100) . "',
                        `ontrx`   = '" . (($ontrx) / 100) . "',
                        `onttvrx` = '" . (($onttvrx) / 100) . "',
                        `oltrx`   = '" . (($oltrx - 10000) / 100) . "'
                    WHERE `index` = '{$device_values['index']}' AND `device` = '{$device_values['device']}';");
    }

?>
